#
# Copyright (C) 2017 Kevin Thornton <krthornt@uci.edu>
#
# This file is part of fwdpy11.
#
# fwdpy11 is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# fwdpy11 is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with fwdpy11.  If not, see <http://www.gnu.org/licenses/>.
#
import unittest
import fwdpy11 as fp11


class testSlocusPop(unittest.TestCase):
    @classmethod
    def setUpClass(self):
        self.pop = fp11.SlocusPop(1000)

    def test_N(self):
        self.assertEqual(self.pop.N, 1000)

    def test_generation(self):
        self.assertEqual(self.pop.generation, 0)

    def test_fitnesses(self):
        # All fitnesses should be 1
        self.assertEqual(
            sum([i.w for i in self.pop.diploid_metadata]),
            float(self.pop.N))

    def test_labels(self):
        # All diploids should be labeled 0 to pop.N-1
        self.assertEqual(
            [i.label for i in self.pop.diploid_metadata],
            [i for i in range(self.pop.N)])

    def test_genetic_values(self):
        self.assertEqual(sum([i.g for i in self.pop.diploid_metadata]), 0.0)

    def test_e_values(self):
        self.assertEqual(sum([i.e for i in self.pop.diploid_metadata]), 0.0)


class testSlocusPopExceptions(unittest.TestCase):
    def testNzero(self):
        with self.assertRaises(ValueError):
            fp11.SlocusPop(0)


class testSampling(unittest.TestCase):
    @classmethod
    def setUpClass(self):
        from quick_pops import quick_nonneutral_slocus
        self.pop = quick_nonneutral_slocus()
        self.rng = fp11.GSLrng(42)

    def testDefinedSample(self):
        self.pop.sample(individuals=range(10))

        with self.assertRaises(IndexError):
            """
            Internally, we should catch cases where i >= N
            """
            self.pop.sample(individuals=range(self.pop.N, self.pop.N + 10))

        with self.assertRaises(Exception):
            """
            pybind11 disallows conversion of negative
            numbers to a list of unsigned types.
            """
            self.pop.sample(individuals=range(-10, 10))


class testPythonObjects(unittest.TestCase):
    @classmethod
    def setUp(self):
        from quick_pops import quick_slocus_qtrait_pop_params
        self.pop, self.pdict = quick_slocus_qtrait_pop_params()
        self.rng = fp11.GSLrng(101)

    def testParentalData(self):
        from fwdpy11.model_params import ModelParams
        from fwdpy11.wright_fisher import evolve
        params = ModelParams(**self.pdict)
        evolve(self.rng, self.pop, params)
        parents = [i.parents for i in self.pop.diploid_metadata]
        for i in parents:
            self.assertTrue(i is not None)
            self.assertTrue(len(i) == 2)
            self.assertTrue(i[0] < self.pop.N)
            self.assertTrue(i[1] < self.pop.N)

    def test_nodes(self):
        import fwdpy11.ts
        self.assertEqual(len(self.pop.diploids),
                         len(self.pop.diploid_metadata))
        for i in self.pop.diploid_metadata:
            self.assertEqual(i.nodes[0], fwdpy11.ts.NULL_NODE)
            self.assertEqual(i.nodes[1], fwdpy11.ts.NULL_NODE)

    def test_nodes_after_evolution(self):
        from fwdpy11.model_params import ModelParams
        from fwdpy11.wright_fisher import evolve
        import fwdpy11.ts
        params = ModelParams(**self.pdict)
        evolve(self.rng, self.pop, params)
        for i in self.pop.diploid_metadata:
            self.assertEqual(i.nodes[0], fwdpy11.ts.NULL_NODE)
            self.assertEqual(i.nodes[1], fwdpy11.ts.NULL_NODE)

    def testMutationLookupTable(self):
        from fwdpy11.model_params import ModelParams
        from fwdpy11.wright_fisher import evolve
        params = ModelParams(**self.pdict)
        evolve(self.rng, self.pop, params)
        lookup = self.pop.mut_lookup
        for i in range(len(self.pop.mcounts)):
            if self.pop.mcounts[i] > 0:
                self.assertTrue(self.pop.mutations[i].pos in lookup)
                self.assertTrue(i in lookup[self.pop.mutations[i].pos])

    def testMutationIndices(self):
        from fwdpy11.model_params import ModelParams
        from fwdpy11.wright_fisher import evolve
        params = ModelParams(**self.pdict)
        evolve(self.rng, self.pop, params)
        lookup = self.pop.mut_lookup
        for key, val in lookup.items():
            indexes = self.pop.mutation_indexes(key)
            self.assertTrue(indexes is not None)
            for i in indexes:
                self.assertTrue(i in val)

    def testEmptyMutationLookupTable(self):
        """
        This test does not use the class fixture.
        Instead, we use an empty pop object.
        """
        pop = fp11.SlocusPop(100)
        self.assertTrue(pop.mut_lookup is None)


if __name__ == "__main__":
    unittest.main()



"""
Plots the mean genetic value for all time periods
present in the tree sequence.

Usage: python3 plot_genetic_values_from_tree_sequence.py filename

filename should contain a pickled population output by SlocusPopGSSmo.py
"""
import pickle
import lzma
import sys
import numpy as np
import seaborn as sns

with lzma.open(sys.argv[1], 'rb') as f:
    pop = pickle.load(f)

nodes = np.array(pop.tables.nodes, copy=False)

ancient_sample_metadata = np.array(pop.ancient_sample_metadata, copy=False)
alive_sample_metadata = np.array(pop.diploid_metadata, copy=False)
metadata = np.hstack((ancient_sample_metadata, alive_sample_metadata))

metadata_times = nodes['time'][metadata['nodes'][:, 0]]

mean_trait_values = []
for u in np.unique(metadata_times):
    samples_at_time_u = np.where(metadata_times == u)[0]
    mean_trait_values.append(
        metadata['g'][samples_at_time_u].mean())

# plot the time vs trait value
xyplot = sns.lineplot(np.unique(metadata_times), mean_trait_values)
xyplot.set_xlabel("Time (generations")
xyplot.set_ylabel("Mean trait value")
xyplot.get_figure().savefig("mean_trait_value_from_tree_sequences.png",
                            dpi=300)



import unittest
import fwdpy11 as fp11
import pyximport
import numpy as np
pyximport.install(setup_args={'include_dirs': np.get_include()})  # noqa
from MeanFitness import MeanFitness


class GenerationRecorder(object):
    def __init__(self):
        self.generations = []

    def __call__(self, pop):
        self.generations.append(pop.generation)


class testWFevolve(unittest.TestCase):
    @classmethod
    def setUpClass(self):
        from fwdpy11.model_params import ModelParams
        from fwdpy11.genetic_values import SlocusMult
        self.pop = fp11.SlocusPop(1000)
        self.rng = fp11.GSLrng(42)
        self.recorder = GenerationRecorder()
        self.cython_recorder = MeanFitness()
        self.p = ModelParams()
        self.p.rates = (1e-3, 1e-3, 1e-3)
        self.p.demography = np.array([1000] * 100, dtype=np.uint32)
        self.p.nregions = [fp11.Region(0, 1, 1)]
        self.p.sregions = [fp11.ExpS(0, 1, 1, -1e-2)]
        self.p.recregions = self.p.nregions
        self.p.gvalue = SlocusMult(2.0)

    def testEvolve(self):
        from fwdpy11.wright_fisher import evolve
        evolve(self.rng, self.pop, self.p, self.recorder)
        self.assertEqual(self.recorder.generations,
                         [i + 1 for i in range(100)])
        self.p.demography = np.array([self.pop.N] * 24)
        evolve(self.rng, self.pop, self.p, self.recorder)
        self.assertEqual(self.recorder.generations,
                         [i + 1 for i in range(124)])


class testCythonRecorder(unittest.TestCase):
    @classmethod
    def setUpClass(self):
        from fwdpy11.model_params import ModelParams
        from fwdpy11.genetic_values import SlocusMult
        self.pop = fp11.SlocusPop(1000)
        self.rng = fp11.GSLrng(42)
        self.cython_recorder = MeanFitness()
        self.p = ModelParams()
        self.p.rates = (1e-3, 1e-3, 1e-3)
        self.p.demography = np.array([1000] * 100, dtype=np.uint32)
        self.p.nregions = [fp11.Region(0, 1, 1)]
        self.p.sregions = [fp11.ExpS(0, 1, 1, -1e-2)]
        self.p.recregions = self.p.nregions
        self.p.gvalue = SlocusMult(2.0)

    def testEvolve(self):
        from fwdpy11.wright_fisher import evolve
        evolve(self.rng, self.pop, self.p, self.cython_recorder)


if __name__ == "__main__":
    unittest.main()



import unittest
import fwdpy11.ts_from_tskit
import numpy as np
import fwdpy11


class testConversion(unittest.TestCase):
    @classmethod
    def setUp(self):
        import msprime
        self.ts = msprime.simulate(10, recombination_rate=0.025, Ne=1000)

    def testGetTables(self):
        n, e, ntips, l = fwdpy11.ts_from_tskit._convert_tables(self.ts)
        self.assertEqual(ntips, 10)
        self.assertEqual(l, 1.0)
        na = np.array(n, copy=False)
        self.assertTrue(np.all(na['time'][:10] == 0))

    # def testGetTablesDiscretizeTime(self):
    #     n, e, ntips, l = fwdpy11.ts_from_msprime._convert_tables(self.ts)
    #     self.assertEqual(ntips, 10)
    #     self.assertEqual(l, 1.0)
    #     na = np.array(n, copy=False)
    #     tzero = np.where(na['time'] == 0.0)
    #     self.assertTrue(len(tzero[0]) == 10)

    def testCreateSlocusPop(self):
        pop = fwdpy11.SlocusPop.create_from_tskit(self.ts)
        self.assertEqual(pop.N, 5)
        self.assertEqual(pop.tables.genome_length(), 1.0)


if __name__ == "__main__":
    unittest.main()



#
# Copyright (C) 2017 Kevin Thornton <krthornt@uci.edu>
#
# This file is part of fwdpy11.
#
# fwdpy11 is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# fwdpy11 is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with fwdpy11.  If not, see <http://www.gnu.org/licenses/>.
#


def evolve(rng, pop, params, recorder=None):
    """
    Evolve a population

    :param rng: random number generator
    :type rng: :class:`fwdpy11.GSLrng`
    :param pop: A population
    :type pop: :class:`fwdpy11.SlocusPop`
    :param params: simulation parameters
    :type params: :class:`fwdpy11.model_params.ModelParams`
    :param recorder: (None) A temporal sampler/data recorder.
    :type recorder: callable

    .. note::
        If recorder is None,
        then :class:`fwdpy11.temporal_samplers.RecordNothing` will be used.

    """
    import fwdpy11.SlocusPop
    import fwdpy11.MlocusPop
    import warnings
    # Test parameters while suppressing warnings
    with warnings.catch_warnings():
        warnings.simplefilter("ignore")
        # Will throw exception if anything is wrong:
        params.validate()

    from fwdpy11 import MutationRegions
    from fwdpy11 import RecombinationRegions
    if pop.__class__ is fwdpy11.SlocusPop:
        from .wright_fisher_slocus import WFSlocusPop
        from fwdpy11 import GeneralizedGeneticMap
        pneutral = params.mutrate_n/(params.mutrate_n+params.mutrate_s)
        mm = MutationRegions.create(pneutral, params.nregions, params.sregions)
        if all([i.__class__ is fwdpy11.Region for i in params.recregions]) is True:
            rm = RecombinationRegions(params.recrate, params.recregions)
        else:
            rm = GeneralizedGeneticMap(params.recregions)

        if recorder is None:
            from fwdpy11.temporal_samplers import RecordNothing
            recorder = RecordNothing()

        WFSlocusPop(rng, pop, params.demography,
                    params.mutrate_n, params.mutrate_s,
                    params.recrate, mm, rm, params.gvalue,
                    recorder, params.pself, params.prune_selected)
    else:
        from .wright_fisher_mlocus import WFMlocusPop
        from fwdpy11 import MlocusMutationRegions
        from fwdpy11 import MlocusRecombinationRegions
        mm = MlocusMutationRegions()
        for n, s, i, j in zip(params.mutrate_n,
                              params.mutrate_s,
                              params.nregions,
                              params.sregions):
            pn = n/(n+s)
            temp = MutationRegions.create(pn, i, j)
            mm.append(temp)
        rm = MlocusRecombinationRegions()
        for i, j in zip(params.recrates, params.recregions):
            rm.append(RecombinationRegions(i, j))
        if recorder is None:
            from fwdpy11.temporal_samplers import RecordNothing
            recorder = RecordNothing()

        WFMlocusPop(rng, pop, params.demography, params.mutrates_n,
                    params.mutrates_s, mm, rm, params.interlocus_rec,
                    params.gvalue, recorder, params.pself,
                    params.prune_selected)



#
# Copyright (C) 2017 Kevin Thornton <krthornt@uci.edu>
#
# This file is part of fwdpy11.
#
# fwdpy11 is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# fwdpy11 is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with fwdpy11.  If not, see <http://www.gnu.org/licenses/>.
#
from fwdpy11._Populations import _MlocusPop


class MlocusPop(_MlocusPop):
    """
    Representation of a multi-locus system.
    """
    @staticmethod
    def create(diploids, gametes, mutations, locus_boundaries, *args):
        """
        Create a new object from input data.
        Unlike the constructor method, this method results
        in no temporary copies of input data.

        :param diplods: A :class:`fwdpy11.VecVecDiploid`
        :param gametes: A :class:`fwdpy11.VecGamete`
        :param mutations: A :class:`fwdpy11.VecMutation`
        :param locus_boundaries: Positional boundaries of each locus
        :param args: Fixations, fixation times, and generation

        :rtype: :class:`fwdpy11.MlocusPop`

        See :ref:`popobjects` for example use.

        When passing in extra args, they must be the following:

        fixations: A :class:`fwdpy11.VecMutation`
        fixation times: A :class:`fwdpy11.VecUint32`
        generation: A non-negative integer

        It is required that len(fixations) == len(fixation times).

        The result of passing in these extra args will be an object
        with its fixation data populated and its generation set
        to the input value.

        .. versionadded:: 0.1.4

        """
        return MlocusPop(_MlocusPop.create(diploids, gametes, mutations, locus_boundaries, args))



"""
"""
import pickle
import lzma
import sys
import numpy as np
import seaborn as sns
import fwdpy11.ts

with lzma.open(sys.argv[1], 'rb') as f:
    pop = pickle.load(f)

nodes = np.array(pop.tables.nodes, copy=False)

ancient_sample_metadata = np.array(pop.ancient_sample_metadata, copy=False)
alive_sample_metadata = np.array(pop.diploid_metadata, copy=False)
metadata = np.hstack((ancient_sample_metadata, alive_sample_metadata))

metadata_nodes = metadata['nodes'].flatten()
metadata_node_times = nodes['time'][metadata_nodes]
metadata_record_times = nodes['time'][metadata['nodes'][:, 0]]


genetic_trait_values_from_sim = []
genetic_values_from_ts = []
idx = 0
for u in np.unique(metadata_node_times):
    samples_at_time_u = metadata_nodes[np.where(metadata_node_times == u)]
    vi = fwdpy11.ts.VariantIterator(
        pop.tables, pop.mutations, samples_at_time_u)
    sum_esizes = np.zeros(len(samples_at_time_u))
    for variant in vi:
        g = variant.genotypes
        r = variant.record
        mutant = np.where(g == 1)[0]
        sum_esizes[mutant] += pop.mutations[r.key].s
    ind = int(len(samples_at_time_u)/2)
    temp_gvalues = np.zeros(ind)
    temp_gvalues += sum_esizes[0::2]
    temp_gvalues += sum_esizes[1::2]
    genetic_values_from_ts.extend(temp_gvalues.tolist())
    genetic_trait_values_from_sim.extend(
        metadata['g'][np.where(metadata_record_times == u)[0]].tolist())

# plot the time vs trait value
xyplot = sns.scatterplot(genetic_trait_values_from_sim, genetic_values_from_ts)
xyplot.set_xlabel("From simulation")
xyplot.set_ylabel("From tree sequences")
xyplot.get_figure().savefig("compare_genetic_values.png",
                            dpi=300)



import unittest
import fwdpy11 as fp11
import fwdpy11.multilocus as fp11m
import numpy as np

class testRecombination(unittest.TestCase):
    @classmethod
    def setUp(self):
        self.rng = fp11.GSLrng(42)

    def testMakeBinomial(self):
        v = [0.5] * 5
        x = fp11m.binomial_rec(v)

    def testMakePoisson(self):
        v = [1e-3] * 5
        x = fp11m.poisson_rec(v)


if __name__ == "__main__":
    unittest.main()




#
# Copyright (C) 2017 Kevin Thornton <krthornt@uci.edu>
#
# This file is part of fwdpy11.
#
# fwdpy11 is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# fwdpy11 is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with fwdpy11.  If not, see <http://www.gnu.org/licenses/>.
#


class RecordNothing:
    """
    A temporal sampler that does nothing.
    """
    def __call__(self, pop):
        """
        Returns the C++ callback for this DFE
        """
        return



def setup_mako(cfg):
    # import fwdpy11 so we can find its C++ headers
    import fwdpy11 as fp11
    import subprocess
    import sys
    gsl_prefix = subprocess.check_output(
        ['gsl-config', '--prefix']).strip().decode('utf-8')
    # add fwdpy11 header locations to the include path
    cfg['include_dirs'].extend(
        [fp11.get_includes(), fp11.get_fwdpp_includes(),
         gsl_prefix + str('/include')])
    gsl_libs = subprocess.check_output(
        ['gsl-config', '--libs']).strip().decode('utf-8')
    gsl_libs = gsl_libs.split(' ')
    gsl_runtime_libs = [i.replace('-l', '') for i in gsl_libs[1:]]
    cfg['libraries'].extend(gsl_runtime_libs)
    cfg['extra_link_args'].extend([gsl_libs[0]])
    # On OS X using clang, there is more work to do.  Using gcc on OS X
    # gets rid of these requirements. The specifics sadly depend on how
    # you initially built fwdpy11, and what is below assumes you used
    # the provided setup.py + OS X + clang:
    if sys.platform == 'darwin':
        cfg['compiler_args'].extend(
            ['-stdlib=libc++', '-mmacosx-version-min=10.7'])
        cfg['linker_args'] = ['-stdlib=libc++', '-mmacosx-version-min=10.7']



# We use cppimport to build the module.
# For the sake of unit testing, we force
# a rebuild every time, but that clearly
# wouldn't be needed for normal use.
import cppimport
import unittest
import pickle
import numpy as np
import fwdpy11 as fp11
import fwdpy11.model_params
import fwdpy11.genetic_values
import fwdpy11.wright_fisher
import fwdpy11.ezparams
cppimport.force_rebuild()
cppimport.set_quiet(False)
snowdrift = cppimport.imp("snowdrift")


class SamplePhenotypes(object):
    """
    Temporal sampler checks that one can hook
    a stateful fitness model to a sampler
    and access its data and that the data
    are as expected.
    """

    def __init__(self, f):
        self.f = f
        # self.a = fwdpy11.genetic_values.SlocusAdditive(2.0)

    def __call__(self, pop):
        pass
        # for i in range(pop.N):
        #     w = self.a(pop.diploids[i], pop)
        #     assert(w == self.f.phenotypes[i])


def evolve_snowdrift(args):
    """
    We write the function taking a tuple
    out of habit, simplifying later
    integration with multiprocessing or
    concurrent.futures.
    """
    N, seed = args
    # Construct as single-deme object
    # with N diploids
    pop = fp11.SlocusPop(N)
    # Initialize a random number generator
    rng = fp11.GSLrng(seed)
    p = {'sregions': [fp11.ExpS(0, 1, 1, -0.1, 1.0)],
         'recregions': [fp11.Region(0, 1, 1)],
         'nregions': [],
         'gvalue': snowdrift.SlocusSnowdrift(0.2, -0.2, 1, -2),
         # evolve for 100 generations so that unit tests are
         # fast
         'demography': np.array([N] * 100, dtype=np.uint32),
         'rates': (0.0, 0.0025, 0.001),
         'prune_selected': False
         }
    params = fwdpy11.model_params.ModelParams(**p)
    sampler = SamplePhenotypes(params.gvalue)
    fp11.wright_fisher.evolve(rng, pop, params, sampler)
    # return our pop
    return pop


class testSnowdrift(unittest.TestCase):
    @classmethod
    def setUp(self):
        self.f = snowdrift.SlocusSnowdrift(1, -1, 0.1, 0.2)

    def testShape(self):
        s = self.f.shape
        self.assertEqual(len(s), 1)
        self.assertEqual(s[0], 1)

    def test_genetic_values(self):
        self.assertEqual(len(self.f.genetic_values), 1)

    def testPickle(self):
        self.f.phenotypes = [1, 2, 3, 4]
        p = pickle.dumps(self.f, -1)
        up = pickle.loads(p)
        self.assertEqual(up.phenotypes, self.f. phenotypes)

    def test_evolve(self):
        p = evolve_snowdrift((1000, 42))
        p


if __name__ == "__main__":
    unittest.main()



from cython.view cimport array as cvarray

cpdef double mean_s(double[:] arr):
    cdef double rv = 0.0
    cdef size_t i = 0
    cdef size_t alen = arr.shape[0]
    while i < alen:
        rv += arr[i]
        i+=1
    return rv/<double>(alen)
    




import fwdpy11
import cppimport
cppimport.force_rebuild()
gsl = cppimport.imp("gsl_error")
import unittest


class testGSLerror(unittest.TestCase):
    def testErrorHandler(self):
        with self.assertRaises(RuntimeError):
            gsl.trigger_error()
        try:
            gsl.trigger_error()
        except RuntimeError as e:
            self.assertEqual(str(e), "matrix not square")

    def testUnhandledGSLError(self):
        with self.assertRaises(RuntimeError):
            gsl.trigger_error_not_handled()


if __name__ == "__main__":
    unittest.main()



#
# Copyright (C) 2017 Kevin Thornton <krthornt@uci.edu>
#
# This file is part of fwdpy11.
#
# fwdpy11 is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# fwdpy11 is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with fwdpy11.  If not, see <http://www.gnu.org/licenses/>.
#

import unittest
import fwdpy11


class testSlocusPopAddMutations(unittest.TestCase):
    @classmethod
    def setUp(self):
        self.pop = fwdpy11.SlocusPop(1000)
        self.mpop = fwdpy11.MlocusPop(1000, [(0, 1), (1, 2)])
        self.mvec = fwdpy11.VecMutation()
        self.nmut = fwdpy11.Mutation(0.1, 0.0, 0.0, 0, 0)
        self.smut = fwdpy11.Mutation(1.2, -0.01, 0.0, 0, 0)
        self.smut_vec = fwdpy11.Mutation(1.3, 0., 0., 0, [-1., 0], [1, 1], 0)

    def testAddOneNeutralMutation(self):
        self.mvec.append(self.nmut)
        m = self.pop.add_mutations(self.mvec, [0], [0])
        self.assertEqual(len(m), 1)
        for i in m:
            self.assertEqual(self.pop.mutations[i].neutral, True)
            self.assertEqual(self.pop.mcounts[i], 1)
        self.assertEqual(
            len(self.pop.gametes[self.pop.diploids[0].first].mutations), 1)
        self.assertEqual(
            len(self.pop.gametes[self.pop.diploids[0].first].smutations), 0)
        self.assertEqual(
            len(self.pop.gametes[self.pop.diploids[0].second].mutations), 0)
        self.assertEqual(
            len(self.pop.gametes[self.pop.diploids[0].second].smutations), 0)

    def testAddOneSelectedMutation(self):
        self.mvec.append(self.smut)
        m = self.pop.add_mutations(self.mvec, [0], [2])
        self.assertEqual(len(m), 1)
        for i in m:
            self.assertEqual(self.pop.mutations[i].neutral, False)
            self.assertEqual(self.pop.mcounts[i], 2)
        self.assertEqual(
            len(self.pop.gametes[self.pop.diploids[0].first].mutations), 0)
        self.assertEqual(
            len(self.pop.gametes[self.pop.diploids[0].first].smutations), 1)
        self.assertEqual(
            len(self.pop.gametes[self.pop.diploids[0].second].mutations), 0)
        self.assertEqual(
            len(self.pop.gametes[self.pop.diploids[0].second].smutations), 1)

    def testAddOneSelectedMutationWithVecEffects(self):
        self.mvec.append(self.smut_vec)
        m = self.pop.add_mutations(self.mvec, [0], [2])

        # The addition of mutations to populations works
        # via move construction, meaning the vector contents
        # of the input must be empty:
        self.assertEqual(len(self.mvec[0].esizes), 0)
        self.assertEqual(len(self.mvec[0].heffects), 0)

        self.assertEqual(len(m), 1)
        for i in m:
            self.assertEqual(self.pop.mutations[i].neutral, False)
            self.assertEqual(self.pop.mcounts[i], 2)
        self.assertEqual(
            len(self.pop.gametes[self.pop.diploids[0].first].mutations), 0)
        self.assertEqual(
            len(self.pop.gametes[self.pop.diploids[0].first].smutations), 1)
        self.assertEqual(
            len(self.pop.gametes[self.pop.diploids[0].second].mutations), 0)
        self.assertEqual(
            len(self.pop.gametes[self.pop.diploids[0].second].smutations), 1)

    def testAddMultipleMutationsSimple(self):
        self.mvec.append(self.smut)
        self.mvec.append(self.nmut)
        # Add mut to every other individual,
        # and they are all homozygous
        ind = [i for i in range(0, self.pop.N, 2)]
        m = self.pop.add_mutations(self.mvec, ind, [2] * len(ind))
        for i in m:
            self.assertEqual(self.pop.mcounts[i], 2 * len(ind))

    def testAddMultipleMutationsComplex(self):
        self.mvec.append(self.smut)
        self.mvec.append(self.nmut)
        ind1 = [i for i in range(0, int(self.pop.N / 2), 2)]
        ind2 = [i for i in range(int(self.pop.N / 2), self.pop.N, 3)]
        c1 = [0] * len(ind1)
        c2 = [1] * len(ind2)

        x = self.pop.add_mutations(self.mvec, ind1 + ind2, c1 + c2)
        self.assertEqual(len(x), 2)
        self.assertEqual(len(self.pop.mutations), 2)
        for i in x:
            self.assertEqual(self.pop.mcounts[i], len(ind1) + len(ind2))
        for i in ind1:
            self.assertEqual(
                len(self.pop.gametes[self.pop.diploids[i].first].mutations), 1)
            self.assertEqual(
                len(self.pop.gametes[self.pop.diploids[i].first].smutations), 1)
            self.assertEqual(
                len(self.pop.gametes[self.pop.diploids[i].second].mutations), 0)
            self.assertEqual(
                len(self.pop.gametes[self.pop.diploids[i].second].smutations), 0)
        for i in ind2:
            self.assertEqual(
                len(self.pop.gametes[self.pop.diploids[i].first].mutations), 0)
            self.assertEqual(
                len(self.pop.gametes[self.pop.diploids[i].first].smutations), 0)
            self.assertEqual(
                len(self.pop.gametes[self.pop.diploids[i].second].mutations), 1)
            self.assertEqual(
                len(self.pop.gametes[self.pop.diploids[i].second].smutations), 1)

    def testAddSameMutationPosition(self):
        self.mvec.append(self.nmut)
        m = self.pop.add_mutations(self.mvec, [0], [0])
        self.mvec = fwdpy11.VecMutation()
        self.mvec.append(self.nmut)
        with self.assertRaises(ValueError):
            m = self.pop.add_mutations(self.mvec, [0], [0])

    def testAddMlocus(self):
        self.mvec.append(self.nmut)
        self.mvec.append(self.smut)
        m = self.mpop.add_mutations(self.mvec, [0], [0])
        d = self.mpop.diploids[0]
        g1l0 = self.mpop.gametes[d[0].first]
        g2l0 = self.mpop.gametes[d[0].second]
        g1l1 = self.mpop.gametes[d[1].first]
        g2l1 = self.mpop.gametes[d[1].second]

        self.assertEqual(len(g1l0.mutations), 1)
        self.assertEqual(len(g1l0.smutations), 0)
        self.assertEqual(len(g2l0.mutations), 0)
        self.assertEqual(len(g2l0.smutations), 0)
        self.assertEqual(len(g1l1.mutations), 0)
        self.assertEqual(len(g1l1.smutations), 1)
        self.assertEqual(len(g2l1.mutations), 0)
        self.assertEqual(len(g2l1.smutations), 0)

    # Test errors that we expect from fwdpp.
    # Note: these should all be ValueError,
    # as they reflect bad input.  We should fix
    # that upstream.

    def testUnEqualListLengths(self):
        self.mvec.append(self.nmut)
        with self.assertRaises(ValueError):
            m = self.pop.add_mutations(self.mvec, [0], [0, 1])

    def testInvalidIndividual(self):
        self.mvec.append(self.nmut)
        with self.assertRaises(IndexError):
            m = self.pop.add_mutations(self.mvec, [self.pop.N], [0])

    def testInvalidGamete(self):
        self.mvec.append(self.nmut)
        with self.assertRaises(IndexError):
            m = self.pop.add_mutations(self.mvec, [53], [3])


if __name__ == "__main__":
    unittest.main()



#
# Copyright (C) 2017 Kevin Thornton <krthornt@uci.edu>
#
# This file is part of fwdpy11.
#
# fwdpy11 is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# fwdpy11 is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with fwdpy11.  If not, see <http://www.gnu.org/licenses/>.
#

"""
    Define classes for parameterizing simulations.

    The classes take either kwargs or an expanded dict as arguments.
    The names of the kwargs are the same as the names of class properties.
    The properties themselves are gettable/settable in the usual way.

    Each class has a validate function that attempts to do a
    full error-checking on the data stored in a class instance.

    The module fwdpy11.ezparams contains some functions for rapidly setting up
    dictionaries of parameters for common cases.

    .. todo::
        The classes should make deep copies of the input so that data
        from the calling environment are not changed, which can happen
        in some cases.
        There is an issue for C++ types holding
        things like Python objects.  Currently,
        tests/test_python_genetic_values.py
        will segfault if we copy.deepcopy the params objects.
"""


def _validate_types(data, typename, strict):
    for i in data:
        if (strict is True and type(i) is not typename) or \
           isinstance(i, typename) is False:
            raise TypeError("invalid type: " + str(typename))


class ModelParams(object):
    """
    Class to hold simulation parameters

    .. versionadded:: 0.1.1

    .. versionchanged:: 0.2.0
        Changed this from a horrible class hierarchy
        into a much simpler, single class.

    See :ref:`model_params` for detailed documentation.
    """

    def __init__(self, **kwargs):
        self.__nregions = []
        self.__sregions = None
        self.__recregions = None
        self.__interlocus_rec = None
        self.__demography = None
        self.__prune_selected = True
        self.__rates = None
        self.__gvalue = None
        self.__pself = 0.0
        for key, value in kwargs.items():
            if key in dir(self) and key[:1] != "_":
                setattr(self, key, value)
            elif key not in dir(self):
                raise ValueError(key, " not a valid parameter for this class.")

    @property
    def nregions(self):
        """
        Get or set the neutral regions.
        """
        return self.__nregions

    @nregions.setter
    def nregions(self, nregions_):
        self.__nregions = nregions_

    @property
    def sregions(self):
        """
        Get or set the selected regions.
        """
        return self.__sregions

    @sregions.setter
    def sregions(self, sregions_):
        self.__sregions = sregions_

    @property
    def recregions(self):
        """
        Get or set the recombination regions.
        """
        return self.__recregions

    @recregions.setter
    def recregions(self, recregions_):
        self.__recregions = recregions_

    @property
    def prune_selected(self):
        """
        Get or set whether or not selected fixations
        are to be removed during simulaton.

        When setting, a bool is required.
        """
        return self.__prune_selected

    @prune_selected.setter
    def prune_selected(self, value):
        self.__prune_selected = bool(value)

    @nregions.setter
    def nregions(self, nregions):
        self.__nregions = nregions

    @sregions.setter
    def sregions(self, sregions):
        self.__sregions = sregions

    @property
    def demography(self):
        """
        Get or set demographic history.
        """
        return self.__demography

    @demography.setter
    def demography(self, value):
        self.__demography = value

    @property
    def gvalue(self):
        """
        Get/set the type name of the genetic value
        calculator.
        """
        return self.__gvalue

    @gvalue.setter
    def gvalue(self, gvalue_):
        self.__gvalue = gvalue_

    @property
    def rates(self):
        """
        Get/set mutation and recombination rates.
        """
        return self.__rates

    @rates.setter
    def rates(self, rates_):
        if len(rates_) != 3:
            raise ValueError("length of rates must be 3")
        self.__rates = rates_

    @property
    def mutrate_n(self):
        """
        Return neutral mutation rate(s).
        """
        return self.__rates[0]

    @property
    def mutrate_s(self):
        """
        Return selected mutation rate(s).
        """
        return self.__rates[1]

    @property
    def recrate(self):
        """
        Return recombination rate(s).
        """
        return self.__rates[2]

    @property
    def mutrates_n(self):
        """
        Return neutral mutation rate(s).
        """
        return self.mutrate_n

    @property
    def mutrates_s(self):
        """
        Return selected mutation rate(s).
        """
        return self.mutrate_s

    @property
    def recrates(self):
        """
        Return recombination rate(s).
        """
        return self.recrate

    @property
    def interlocus_rec(self):
        """
        Get/set interlocus recombination functions.
        """
        return self.__interlocus_rec

    @interlocus_rec.setter
    def interlocus_rec(self, ir):
        self.__interlocus_rec = ir

    @property
    def pself(self):
        """
        Get/set selfing probability/probabilities.
        """
        return self.__pself

    @pself.setter
    def pself(self, pself_):
        self.__pself = pself_

    def validate(self):
        """
        Error check model params.

        :raises TypeError: Throws TypeError if validation fails.

        """
        if self.nregions is None:
            raise TypeError("neutral regions cannot be None")
        if self.sregions is None:
            raise TypeError("selected regions cannot be None")
        if self.recregions is None:
            raise TypeError("recombination regions cannot be None")
        if self.demography is None:
            raise TypeError("demography cannot be None")
        if self.prune_selected is None:
            raise TypeError("prune_selected cannot be None")
        if self.gvalue is None:
            raise TypeError("gvalue cannot be None")
        if self.rates is None:
            raise TypeError("rates cannot be None")


def _validate_single_deme_demography(value):
    import numpy as np
    if any(i < 0 for i in value):
        raise ValueError("all population sizes must be > 0")
    if (type(value) is np.ndarray) is False:
        raise ValueError("Type for population size " +
                         "history must be numpy.array")


def _validate_single_locus_rates(data):
    for key, value in data.items():
        if value is None:
            raise ValueError(key + " cannot be None")
        if value < 0.:
            raise ValueError("All mutation and recombination " +
                             "rates must be non-negative.")



#
# Copyright (C) 2017 Kevin Thornton <krthornt@uci.edu>
#
# This file is part of fwdpy11.
#
# fwdpy11 is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# fwdpy11 is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with fwdpy11.  If not, see <http://www.gnu.org/licenses/>.
#
import unittest
import cppimport
cppimport.force_rebuild()
cppimport.set_quiet(False)
pickling_cpp = cppimport.imp("pickling_cpp")


class testPickleMutation(unittest.TestCase):
    @classmethod
    def setUp(self):
        import fwdpy11
        self.m = fwdpy11.Mutation(0.1, -0.2, 1.0, 10, 3)

    def testUnpickle(self):
        import pickle
        o = pickling_cpp.pickle_mutation(self.m)
        m = pickle.loads(o)
        self.assertEqual(m, self.m)

    def testUnpickleFromGeneralPickler(self):
        import pickle
        o = pickling_cpp.general_pickler(self.m)
        m = pickle.loads(o)
        self.assertEqual(m, self.m)


class testPickleSlocusPop(unittest.TestCase):
    @classmethod
    def setUp(self):
        from quick_pops import quick_neutral_slocus
        self.pop = quick_neutral_slocus()

    def testPickleMutations(self):
        import pickle
        p = [pickling_cpp.general_pickler(i) for i in self.pop.mutations]
        for i, j in zip(p, self.pop.mutations):
            m = pickle.loads(i)
            self.assertEqual(m, j)
        x = pickle.dumps(self.pop.mutations)
        p = pickle.loads(x)
        for i, j in zip(p, self.pop.mutations):
            self.assertEqual(i, j)

    def testPickleGametes(self):
        import pickle
        p = [pickling_cpp.general_pickler(i) for i in self.pop.gametes]
        for i, j in zip(p, self.pop.gametes):
            m = pickle.loads(i)
            self.assertEqual(m, j)
        x = pickling_cpp.general_pickler(self.pop.gametes)
        p = pickle.loads(x)
        for i, j in zip(p, self.pop.gametes):
            self.assertEqual(i, j)

    def testPickleDiploidsPy(self):
        import pickle
        pd = pickle.dumps(self.pop.diploids)
        p = pickle.loads(pd)
        for i, j in zip(p, self.pop.diploids):
            self.assertEqual(i, j)

    def testPickleDiploidsCpp(self):
        import pickle
        p = [pickling_cpp.general_pickler(i) for i in self.pop.diploids]
        for i, j in zip(p, self.pop.diploids):
            m = pickle.loads(i)
            self.assertEqual(m, j)
        x = pickling_cpp.general_pickler(self.pop.diploids)
        p = pickle.loads(x)
        for i, j in zip(p, self.pop.diploids):
            self.assertEqual(i, j)

    def testPicklePopPy(self):
        import pickle
        p = pickle.dumps(self.pop, -1)
        pp = pickle.loads(p)
        self.assertEqual(pp, self.pop)

    def testPicklePopCpp(self):
        import pickle
        p = pickling_cpp.general_pickler(self.pop)
        pp = pickle.loads(p)
        self.assertEqual(pp, self.pop)

    def testPickleTableCollection(self):
        import pickle
        import numpy as np
        p = pickle.dumps(self.pop.tables)
        up = pickle.loads(p)
        self.assertEqual(up.genome_length(), np.finfo(np.float).max)


class testPickleSlocusPopTreeSequences(unittest.TestCase):
    @classmethod
    def setUp(self):
        import fwdpy11.genetic_values
        import fwdpy11.wright_fisher_ts
        import numpy as np
        self.N = 1000
        self.demography = np.array([self.N]*self.N, dtype=np.uint32)
        self.rho = 1.
        self.theta = 100.
        self.nreps = 500
        self.mu = self.theta/(4*self.N)
        self.r = self.rho/(4*self.N)

        self.GSS = fwdpy11.genetic_values.GSS(VS=1, opt=0)
        a = fwdpy11.genetic_values.SlocusAdditive(2.0, self.GSS)
        self.p = {'nregions': [],
                  'sregions': [fwdpy11.GaussianS(0, 1, 1, 0.25)],
                  'recregions': [fwdpy11.Region(0, 1, 1)],
                  'rates': (0.0, 0.025, self.r),
                  'gvalue': a,
                  'prune_selected': False,
                  'demography': self.demography
                  }
        self.params = fwdpy11.model_params.ModelParams(**self.p)
        self.rng = fwdpy11.GSLrng(101*45*110*210)
        self.pop = fwdpy11.SlocusPop(self.N, 1.0)
        fwdpy11.wright_fisher_ts.evolve(self.rng, self.pop, self.params, 100)

    def testPickleNodeTable(self):
        import pickle
        import numpy as np
        pn = pickle.dumps(self.pop.tables.nodes)
        up = pickle.loads(pn)
        nodes = np.array(self.pop.tables.nodes)
        nodes2 = np.array(up)
        self.assertTrue(np.array_equal(nodes, nodes2))

    def testPickleEdgeTable(self):
        import pickle
        import numpy as np
        p = pickle.dumps(self.pop.tables.edges)
        u = pickle.loads(p)
        edges = np.array(self.pop.tables.edges)
        edges2 = np.array(u)
        self.assertTrue(np.array_equal(edges, edges2))

    def testPickleMutationTable(self):
        import pickle
        import numpy as np
        p = pickle.dumps(self.pop.tables.mutations)
        u = pickle.loads(p)
        mutations = np.array(self.pop.tables.mutations)
        mutations2 = np.array(u)
        self.assertTrue(np.array_equal(mutations, mutations2))

    def testPickleTableCollection(self):
        import pickle
        p = pickle.dumps(self.pop.tables)
        u = pickle.loads(p)
        self.assertTrue(u == self.pop.tables)

    def testPickleMetaData(self):
        import pickle
        import numpy as np
        p = pickle.dumps(self.pop.diploid_metadata)
        up = pickle.loads(p)
        md = np.array(self.pop.diploid_metadata)
        md2 = np.array(up)
        self.assertTrue(np.array_equal(md, md2))

    def testPicklingToFile(self):
        import fwdpy11
        import lzma
        import os
        fname = "pickleFileTesting.pickle"
        with lzma.open(fname, 'wb') as f:
            self.pop.pickle_to_file(f)
        with lzma.open(fname, 'rb') as f:
            pop = fwdpy11.SlocusPop.load_from_pickle_file(f)
        self.assertEqual(pop.N, self.pop.N)
        self.assertEqual(pop.generation, self.pop.generation)
        self.assertTrue(pop.diploids == self.pop.diploids)
        self.assertTrue(pop.gametes == self.pop.gametes)
        self.assertTrue(pop.mutations == self.pop.mutations)
        self.assertTrue(pop.mcounts == self.pop.mcounts)
        self.assertTrue(pop.tables == self.pop.tables)
        self.assertTrue(pop == self.pop)
        os.remove(fname)


class testPickleMlocusPop(unittest.TestCase):
    @classmethod
    def setUp(self):
        from quick_pops import quick_mlocus_qtrait
        self.pop = quick_mlocus_qtrait(N=500, simlen=10)

    def testPickleDiploids(self):
        import pickle
        x = pickle.dumps(self.pop.diploids)
        p = pickle.loads(x)
        for i, j in zip(p, self.pop.diploids):
            self.assertEqual(i, j)

    def testPicklePopPy(self):
        import pickle
        p = pickle.dumps(self.pop, -1)
        pp = pickle.loads(p)
        self.assertEqual(pp, self.pop)

    def testPicklePopCpp(self):
        import pickle
        p = pickling_cpp.general_pickler(self.pop)
        pp = pickle.loads(p)
        self.assertEqual(pp, self.pop)

    def testPicklingToFile(self):
        import fwdpy11
        import lzma
        import os
        fname = "pickleFileTestingMlocus.pickle"
        with lzma.open(fname, 'wb') as f:
            self.pop.pickle_to_file(f)
        with lzma.open(fname, 'rb') as f:
            pop = fwdpy11.MlocusPop.load_from_pickle_file(f)
        self.assertTrue(pop == self.pop)
        self.assertEqual(pop.N, self.pop.N)
        os.remove(fname)


if __name__ == "__main__":
    unittest.main()



import unittest
import fwdpy11 as fp11
import fwdpy11.sampling
import numpy as np
import numpy as np
import pyximport
pyximport.install(setup_args={'include_dirs': np.get_include()})
from mean_sel_coeff import mean_s


class test_SlocusPop(unittest.TestCase):
    """
    Test coercion of population data
    to numpy arrays
    """
    @classmethod
    def setUpClass(self):
        from quick_pops import quick_nonneutral_slocus
        self.pop = quick_nonneutral_slocus()
        self.muts = np.array(self.pop.mutations.array())

    def testMutations(self):
        self.assertTrue(isinstance(self.muts, np.ndarray))
        self.assertEqual(self.muts.shape[0], len(self.pop.mutations))
        for i, j in zip(self.muts, self.pop.mutations):
            self.assertEqual(i['neutral'], j.neutral)
            self.assertEqual(i['h'], j.h)
            self.assertEqual(i['pos'], j.pos)
            self.assertEqual(i['g'], j.g)
            self.assertEqual(i['s'], j.s)

            m = fwdpy11.Mutation(tuple(i)[:-1])
            self.assertEqual(m.pos, j.pos)
            self.assertEqual(m.s, j.s)
            self.assertEqual(m.h, j.h)
            self.assertEqual(m.g, j.g)
            self.assertEqual(m.label, j.label)
            self.assertEqual(m.neutral, j.neutral)

    def testCythonFunc(self):
        ms = mean_s(self.muts['s'])
        self.assertAlmostEqual(ms, self.muts['s'].mean())

    def testDiploidTraits(self):
        dips = np.array(self.pop.diploid_metadata)
        for i, j in zip(dips, self.pop.diploid_metadata):
            self.assertEqual(i['g'], j.g)
            self.assertEqual(i['e'], j.e)
            self.assertEqual(i['w'], j.w)

    def testDiploidTraitsSlice(self):
        dips = np.array(self.pop.diploid_metadata[15: 103: 11])
        for i, j in zip(dips, self.pop.diploid_metadata[15:103:11]):
            self.assertEqual(i['g'], j.g)
            self.assertEqual(i['e'], j.e)
            self.assertEqual(i['w'], j.w)

    def testDiploidKeys(self):
        keys = np.array(self.pop.diploids)
        for i, j in zip(keys, self.pop.diploids):
            self.assertEqual(i['first'], j.first)
            self.assertEqual(i['second'], j.second)


class test_MlocusPop(unittest.TestCase):
    """
    Test coercion of population data
    to numpy arrays
    """
    @classmethod
    def setUpClass(self):
        from quick_pops import quick_mlocus_qtrait
        self.pop = quick_mlocus_qtrait()
        self.muts = np.array(self.pop.mutations.array())

    def testMutations(self):
        self.assertTrue(isinstance(self.muts, np.ndarray))
        self.assertEqual(self.muts.shape[0], len(self.pop.mutations))
        for i, j in zip(self.muts, self.pop.mutations):
            self.assertEqual(i['neutral'], j.neutral)
            self.assertEqual(i['h'], j.h)
            self.assertEqual(i['pos'], j.pos)
            self.assertEqual(i['g'], j.g)
            self.assertEqual(i['s'], j.s)

    def testDiploidTraits(self):
        dips = np.array(self.pop.diploid_metadata)
        for i, j in zip(dips, self.pop.diploid_metadata):
            self.assertEqual(i['g'], j.g)
            self.assertEqual(i['e'], j.e)
            self.assertEqual(i['w'], j.w)

    def testDiploidTraitsSlice(self):
        dips = np.array(self.pop.diploid_metadata[15:103:11])
        for i, j in zip(dips, self.pop.diploid_metadata[15:103:11]):
            self.assertEqual(i['g'], j.g)
            self.assertEqual(i['e'], j.e)
            self.assertEqual(i['w'], j.w)


if __name__ == "__main__":
    unittest.main()



import unittest
import fwdpy11.sampling
import numpy as np
from quick_pops import quick_nonneutral_slocus, quick_mlocus_qtrait


class test_DataMatrixFromSlocusPop(unittest.TestCase):
    """
    Much of this is already tested in fwdpp's unit tests,
    but the purpose here is to make sure that the wrappers
    are working appropriately.
    """
    @classmethod
    def setUpClass(self):
        self.pop = quick_nonneutral_slocus()
        self.indlist = [i for i in range(100, 150)]
        self.keys = fwdpy11.sampling.mutation_keys(self.pop, self.indlist)
        self.hm = fwdpy11.sampling.haplotype_matrix(
            self.pop, self.indlist, self.keys[0], self.keys[1])
        self.gm = fwdpy11.sampling.genotype_matrix(
            self.pop, self.indlist, self.keys[0], self.keys[1])
        self.hm_neutral = np.array(self.hm.neutral)
        self.hm_selected = np.array(self.hm.selected)
        self.gm_neutral = np.array(self.gm.neutral)
        self.gm_selected = np.array(self.gm.selected)

    def testNcol(self):
        self.assertEqual(self.hm.ncol, 2*len(self.indlist))
        self.assertEqual(self.gm.ncol, len(self.indlist))

    def testKeyNeutralityAndCount(self):
        for i in self.keys[0]:
            self.assertTrue(self.pop.mutations[i[0]].neutral)
            self.assertTrue(self.pop.mcounts[i[0]] > 0)
        for i in self.keys[1]:
            self.assertFalse(self.pop.mutations[i[0]].neutral)
            self.assertTrue(self.pop.mcounts[i[0]] > 0)

    def testHapMat(self):
        self.assertTrue(self.hm_neutral.dtype == np.int8)
        self.assertEqual(self.hm_neutral.ndim, 2)
        self.assertTrue(self.hm_selected.dtype == np.int8)
        self.assertEqual(self.hm_selected.ndim, 2)
        # Get the col sums
        colSums = self.hm_neutral.sum(axis=0)
        colSumsSel = self.hm_selected.sum(axis=0)
        self.assertEqual(len(colSums), self.hm_neutral.shape[1])
        self.assertEqual(len(colSumsSel), self.hm_selected.shape[1])
        j = 0
        for i in range(100, 150):
            # Num neutral variants in diploid i, gamete 0
            nmuts = len(self.pop.gametes[self.pop.diploids[i].first].mutations)
            self.assertEqual(nmuts, colSums[j])
            # Num neutral variants in diploid i, gamete 1
            nmuts = len(
                self.pop.gametes[self.pop.diploids[i].second].mutations)
            self.assertEqual(nmuts, colSums[j + 1])

            # Now, test numbers of selected
            nmuts = len(
                self.pop.gametes[self.pop.diploids[i].first].smutations)
            self.assertEqual(nmuts, colSumsSel[j])
            nmuts = len(
                self.pop.gametes[self.pop.diploids[i].second].smutations)
            self.assertEqual(nmuts, colSumsSel[j + 1])

            j += 2

    def testGenoMat(self):
        self.assertTrue(self.gm_neutral.dtype == np.int8)
        self.assertEqual(self.gm_neutral.ndim, 2)
        # Get the col sums
        colSums = self.gm_neutral.sum(axis=0)
        colSumsSel = self.gm_selected.sum(axis=0)
        self.assertEqual(len(colSums), self.gm_neutral.shape[1])
        self.assertEqual(len(colSumsSel), self.gm_selected.shape[1])
        j = 0
        for i in range(100, 150):
            nmuts = len(self.pop.gametes[self.pop.diploids[i].first].mutations)
            nmuts += len(self.pop.gametes[self.pop.diploids[i].second].mutations)
            self.assertEqual(nmuts, colSums[j])

            # Now, test numbers of selected
            nmuts = len(
                self.pop.gametes[self.pop.diploids[i].first].smutations)
            nmuts += len(self.pop.gametes[self.pop.diploids[i].second].smutations)
            self.assertEqual(nmuts, colSumsSel[j])

            j += 1

    def testHapMatToSample(self):
        neut_sample, sel_sample = fwdpy11.sampling.matrix_to_sample(self.hm)
        rowSums = self.hm_neutral.sum(axis=1)
        rowSumsSel = self.hm_selected.sum(axis=1)
        self.assertEqual(len(neut_sample), self.hm_neutral.shape[0])
        self.assertEqual(len(rowSums), len(neut_sample))
        self.assertEqual(len(sel_sample), self.hm_selected.shape[0])
        self.assertEqual(len(rowSumsSel), len(sel_sample))
        i = 0
        for ni in neut_sample:
            num_ones = ni[1].count('1')
            self.assertEqual(num_ones, rowSums[i])
            i += 1


class test_DataMatrixFromMlocusPop(unittest.TestCase):
    @classmethod
    def setUpClass(self):
        self.pop = quick_mlocus_qtrait()
        self.indlist = [i for i in range(100, 150)]
        self.keys = fwdpy11.sampling.mutation_keys(self.pop, self.indlist)
        self.nkeys = sorted(
            self.keys[0], key=lambda x: self.pop.mutations[x[0]].pos)
        self.skeys = sorted(
            self.keys[1], key=lambda x: self.pop.mutations[x[0]].pos)
        self.hm = fwdpy11.sampling.haplotype_matrix(
            self.pop, self.indlist, self.nkeys, self.skeys)
        self.gm = fwdpy11.sampling.genotype_matrix(
            self.pop, self.indlist, self.nkeys, self.skeys)
        self.hm_neutral = np.array(self.hm.neutral)
        self.hm_selected = np.array(self.hm.selected)
        self.gm_neutral = np.array(self.gm.neutral)
        self.gm_selected = np.array(self.gm.selected)

    def testConvertHapMatrixToSample(self):
        nsample, ssample = fwdpy11.sampling.matrix_to_sample(self.hm)
        nsample_split = fwdpy11.sampling.separate_samples_by_loci(
            self.pop.locus_boundaries, nsample)
        self.assertEqual(len(nsample_split), len(self.pop.locus_boundaries))
        self.assertEqual(len(nsample_split), self.pop.nloci)
        self.assertEqual(len(nsample_split), len(self.pop.locus_boundaries))
        key = 0
        for locus_data in nsample_split:
            self.assertTrue(sorted(locus_data, key=lambda x: x[0]))
            for site in locus_data:
                self.assertTrue(site[0] >= self.pop.locus_boundaries[key][0])
                self.assertTrue(site[0] < self.pop.locus_boundaries[key][1])
                self.assertTrue(len(site[1]) == self.hm_neutral.shape[1])
            key += 1

    def testConvertGenoMatrixToSample(self):
        nsample, ssample = fwdpy11.sampling.matrix_to_sample(self.gm)
        nsample_split = fwdpy11.sampling.separate_samples_by_loci(
            self.pop.locus_boundaries, nsample)
        self.assertEqual(len(nsample_split), len(self.pop.locus_boundaries))
        key = 0
        for locus in nsample_split:
            for site in locus:
                self.assertTrue(site[0] >= self.pop.locus_boundaries[key][0])
                self.assertTrue(site[0] < self.pop.locus_boundaries[key][1])
                self.assertTrue(len(site[1]) == self.gm_neutral.shape[1])
            key += 1

    def testSampleByLocus(self):
        lm = self.pop.sample_by_locus(self.indlist)
        self.assertEqual(len(lm), self.pop.nloci)
        dm = self.pop.sample(self.indlist)
        nsample, ssample = fwdpy11.sampling.matrix_to_sample(dm)
        nsample_split = fwdpy11.sampling.separate_samples_by_loci(
            self.pop.locus_boundaries, nsample)
        for i, j in zip(lm, nsample_split):
            # Test that positons are the same
            pi = [k for k in i.neutral.positions]
            pj = [k[0] for k in j]
            self.assertEqual(pi, pj)
            im = np.array(i.neutral)

            # convert sample data to matrix-like data
            temp = []
            for k in j:
                for c in k[1]:
                    if c == '0':
                        temp.append(0)
                    else:
                        temp.append(1)

            tempa = np.array(temp, dtype=im.dtype).reshape(im.shape)
            self.assertTrue(np.array_equal(im, tempa))


if __name__ == "__main__":
    unittest.main()



import unittest
import fwdpy11
import fwdpy11.genetic_values
import numpy as np


class testSlocusAdditive(unittest.TestCase):
    @classmethod
    def setUp(self):
        from fwdpy11.genetic_value_noise import GaussianNoise
        self.w = fwdpy11.genetic_values.SlocusAdditive(2.0)
        self.t = fwdpy11.genetic_values.SlocusAdditive(
            2.0, fwdpy11.genetic_values.GSS(0.0, 1.0))
        self.tn = fwdpy11.genetic_values.SlocusAdditive(1.0,
                                                        fwdpy11.genetic_values.GSS(
                                                            0.0, 1.0),
                                                        GaussianNoise(mean=0.1, sd=2.0))

    def testScaling(self):
        self.assertEqual(self.w.scaling, 2.0)
        self.assertEqual(self.t.scaling, 2.0)
        self.assertEqual(self.tn.scaling, 1.0)

    def testFitnessOrTrait(self):
        self.assertEqual(self.w.is_fitness, True)
        self.assertEqual(self.t.is_fitness, False)
        self.assertEqual(self.tn.is_fitness, False)

    def testPickleFitness(self):
        import pickle
        p = pickle.dumps(self.w)
        up = pickle.loads(p)
        self.assertEqual(up.scaling, self.w.scaling)
        self.assertTrue(up.is_fitness)
        self.assertEqual(type(up.noise), type(self.w.noise))
        self.assertEqual(type(up.gvalue_to_fitness),
                         type(self.w.gvalue_to_fitness))

    def testPickleTraitNoNoise(self):
        import pickle
        p = pickle.dumps(self.t)
        up = pickle.loads(p)
        self.assertEqual(up.scaling, self.t.scaling)
        self.assertTrue(up.is_fitness is False)
        self.assertEqual(type(up.noise), type(self.t.noise))
        self.assertEqual(type(up.gvalue_to_fitness),
                         type(self.t.gvalue_to_fitness))

    def testPickleTraitWithNoise(self):
        import pickle
        p = pickle.dumps(self.tn)
        up = pickle.loads(p)
        self.assertEqual(up.scaling, self.tn.scaling)
        self.assertTrue(up.is_fitness is False)
        self.assertEqual(type(up.noise), type(self.tn.noise))
        self.assertEqual(type(up.gvalue_to_fitness),
                         type(self.tn.gvalue_to_fitness))

    def testPickleTraitWithNoiseToFile(self):
        import pickle
        with open("ptest.pickle", "wb") as f:
            pickle.dump(self.tn, f)

        with open("ptest.pickle", "rb") as f:
            up = pickle.load(f)
        self.assertEqual(up.scaling, self.tn.scaling)
        self.assertTrue(up.is_fitness is False)
        self.assertEqual(type(up.noise), type(self.tn.noise))
        self.assertEqual(type(up.gvalue_to_fitness),
                         type(self.tn.gvalue_to_fitness))


class testMlocusAdditive(unittest.TestCase):
    @classmethod
    def setUp(self):
        from fwdpy11.genetic_value_noise import GaussianNoise
        self.w = fwdpy11.genetic_values.MlocusAdditive(2.0)
        self.t = fwdpy11.genetic_values.MlocusAdditive(
            2.0, fwdpy11.genetic_values.GSS(0.0, 1.0))
        self.tn = fwdpy11.genetic_values.MlocusAdditive(1.0,
                                                        fwdpy11.genetic_values.GSS(
                                                            0.0, 1.0),
                                                        GaussianNoise(mean=0.1, sd=2.0))

    def testScaling(self):
        self.assertEqual(self.w.scaling, 2.0)
        self.assertEqual(self.t.scaling, 2.0)
        self.assertEqual(self.tn.scaling, 1.0)

    def testFitnessOrTrait(self):
        self.assertEqual(self.w.is_fitness, True)
        self.assertEqual(self.t.is_fitness, False)
        self.assertEqual(self.tn.is_fitness, False)

    def testPickleFitness(self):
        import pickle
        p = pickle.dumps(self.w)
        up = pickle.loads(p)
        self.assertEqual(up.scaling, self.w.scaling)
        self.assertTrue(up.is_fitness)
        self.assertEqual(type(up.noise), type(self.w.noise))
        self.assertEqual(type(up.gvalue_to_fitness),
                         type(self.w.gvalue_to_fitness))

    def testPickleTraitNoNoise(self):
        import pickle
        p = pickle.dumps(self.t)
        up = pickle.loads(p)
        self.assertEqual(up.scaling, self.t.scaling)
        self.assertTrue(up.is_fitness is False)
        self.assertEqual(type(up.noise), type(self.t.noise))
        self.assertEqual(type(up.gvalue_to_fitness),
                         type(self.t.gvalue_to_fitness))

    def testPickleTraitWithNoise(self):
        import pickle
        p = pickle.dumps(self.tn)
        up = pickle.loads(p)
        self.assertEqual(up.scaling, self.tn.scaling)
        self.assertTrue(up.is_fitness is False)
        self.assertEqual(type(up.noise), type(self.tn.noise))
        self.assertEqual(type(up.gvalue_to_fitness),
                         type(self.tn.gvalue_to_fitness))


class testSlocusMult(unittest.TestCase):
    @classmethod
    def setUp(self):
        from fwdpy11.genetic_value_noise import GaussianNoise
        self.w = fwdpy11.genetic_values.SlocusMult(2.0)
        self.t = fwdpy11.genetic_values.SlocusMult(
            2.0, fwdpy11.genetic_values.GSS(0.0, 1.0))
        self.tn = fwdpy11.genetic_values.SlocusMult(1.0,
                                                    fwdpy11.genetic_values.GSS(
                                                        0.0, 1.0),
                                                    GaussianNoise(mean=0.1, sd=2.0))

    def testScaling(self):
        self.assertEqual(self.w.scaling, 2.0)
        self.assertEqual(self.t.scaling, 2.0)
        self.assertEqual(self.tn.scaling, 1.0)

    def testFitnessOrTrait(self):
        self.assertEqual(self.w.is_fitness, True)
        self.assertEqual(self.t.is_fitness, False)
        self.assertEqual(self.tn.is_fitness, False)

    def testPickleFitness(self):
        import pickle
        p = pickle.dumps(self.w)
        up = pickle.loads(p)
        self.assertEqual(up.scaling, self.w.scaling)
        self.assertTrue(up.is_fitness)
        self.assertEqual(type(up.noise), type(self.w.noise))
        self.assertEqual(type(up.gvalue_to_fitness),
                         type(self.w.gvalue_to_fitness))

    def testPickleTraitNoNoise(self):
        import pickle
        p = pickle.dumps(self.t)
        up = pickle.loads(p)
        self.assertEqual(up.scaling, self.t.scaling)
        self.assertTrue(up.is_fitness is False)
        self.assertEqual(type(up.noise), type(self.t.noise))
        self.assertEqual(type(up.gvalue_to_fitness),
                         type(self.t.gvalue_to_fitness))

    def testPickleTraitWithNoise(self):
        import pickle
        p = pickle.dumps(self.tn)
        up = pickle.loads(p)
        self.assertEqual(up.scaling, self.tn.scaling)
        self.assertTrue(up.is_fitness is False)
        self.assertEqual(type(up.noise), type(self.tn.noise))
        self.assertEqual(type(up.gvalue_to_fitness),
                         type(self.tn.gvalue_to_fitness))


class testMlocusMult(unittest.TestCase):
    @classmethod
    def setUp(self):
        from fwdpy11.genetic_value_noise import GaussianNoise
        self.w = fwdpy11.genetic_values.MlocusMult(2.0)
        self.t = fwdpy11.genetic_values.MlocusMult(
            2.0, fwdpy11.genetic_values.GSS(0.0, 1.0))
        self.tn = fwdpy11.genetic_values.MlocusMult(1.0,
                                                    fwdpy11.genetic_values.GSS(
                                                        0.0, 1.0),
                                                    GaussianNoise(mean=0.1, sd=2.0))

    def testScaling(self):
        self.assertEqual(self.w.scaling, 2.0)
        self.assertEqual(self.t.scaling, 2.0)
        self.assertEqual(self.tn.scaling, 1.0)

    def testFitnessOrTrait(self):
        self.assertEqual(self.w.is_fitness, True)
        self.assertEqual(self.t.is_fitness, False)
        self.assertEqual(self.tn.is_fitness, False)

    def testPickleFitness(self):
        import pickle
        p = pickle.dumps(self.w)
        up = pickle.loads(p)
        self.assertEqual(up.scaling, self.w.scaling)
        self.assertTrue(up.is_fitness)
        self.assertEqual(type(up.noise), type(self.w.noise))
        self.assertEqual(type(up.gvalue_to_fitness),
                         type(self.w.gvalue_to_fitness))

    def testPickleTraitNoNoise(self):
        import pickle
        p = pickle.dumps(self.t)
        up = pickle.loads(p)
        self.assertEqual(up.scaling, self.t.scaling)
        self.assertTrue(up.is_fitness is False)
        self.assertEqual(type(up.noise), type(self.t.noise))
        self.assertEqual(type(up.gvalue_to_fitness),
                         type(self.t.gvalue_to_fitness))

    def testPickleTraitWithNoise(self):
        import pickle
        p = pickle.dumps(self.tn)
        up = pickle.loads(p)
        self.assertEqual(up.scaling, self.tn.scaling)
        self.assertTrue(up.is_fitness is False)
        self.assertEqual(type(up.noise), type(self.tn.noise))
        self.assertEqual(type(up.gvalue_to_fitness),
                         type(self.tn.gvalue_to_fitness))


class testSlocusGBR(unittest.TestCase):
    @classmethod
    def setUp(self):
        self.gss = fwdpy11.genetic_values.GSS(0.0, 1.0)
        self.gnoise = fwdpy11.genetic_value_noise.GaussianNoise(
            mean=0.0, sd=1.0)
        self.nonoise = fwdpy11.genetic_value_noise.NoNoise()

    def testPicklingGSS(self):
        import pickle
        gbr = fwdpy11.genetic_values.SlocusGBR(self.gss)
        p = pickle.dumps(gbr, -1)
        up = pickle.loads(p)
        self.assertEqual(type(self.nonoise), type(up.noise))
        self.assertEqual(type(self.gss), type(up.gvalue_to_fitness))

    def testPicklingGSSGaussianNoise(self):
        import pickle
        gbr = fwdpy11.genetic_values.SlocusGBR(self.gss, self.gnoise)
        p = pickle.dumps(gbr, -1)
        up = pickle.loads(p)
        self.assertEqual(type(self.gnoise), type(up.noise))
        self.assertEqual(type(self.gss), type(up.gvalue_to_fitness))


class testMlocusGBR(unittest.TestCase):
    @classmethod
    def setUp(self):
        self.gss = fwdpy11.genetic_values.GSS(0.0, 1.0)
        self.gnoise = fwdpy11.genetic_value_noise.GaussianNoise(
            mean=0.0, sd=1.0)
        self.nonoise = fwdpy11.genetic_value_noise.NoNoise()

    def testPicklingGSS(self):
        import pickle
        gbr = fwdpy11.genetic_values.MlocusGBR(self.gss)
        p = pickle.dumps(gbr, -1)
        up = pickle.loads(p)
        self.assertEqual(type(self.nonoise), type(up.noise))
        self.assertEqual(type(self.gss), type(up.gvalue_to_fitness))

    def testPicklingGSSGaussianNoise(self):
        import pickle
        gbr = fwdpy11.genetic_values.MlocusGBR(self.gss, self.gnoise)
        p = pickle.dumps(gbr, -1)
        up = pickle.loads(p)
        self.assertEqual(type(self.gnoise), type(up.noise))
        self.assertEqual(type(self.gss), type(up.gvalue_to_fitness))


class testGSS(unittest.TestCase):
    @classmethod
    def setUp(self):
        self.x = fwdpy11.genetic_values.GSS(0.0, 1.0)

    def testProperties(self):
        self.assertEqual(self.x.opt, 0.0)
        self.assertEqual(self.x.VS, 1.0)


class testGSSandGSSmoConsistency(unittest.TestCase):
    """
    This tests that GSS and GSSmo have opt and VS
    in the same order.  If that were not true,
    fitness calculations would come out differently
    and the test would fail.
    """
    @classmethod
    def setUp(self):
        self.a = fwdpy11.genetic_values.SlocusAdditive(
            2.0, fwdpy11.genetic_values.GSS(0.0, 1.0))
        self.b = fwdpy11.genetic_values.SlocusAdditive(
            2.0, fwdpy11.genetic_values.GSSmo([(0, 0.0, 1.0)]))
        self.pop = fwdpy11.SlocusPop(1000)

    def testFitnesses(self):
        wa = [self.a.fitness(i, self.pop) for i in range(self.pop.N)]
        wb = [self.b.fitness(i, self.pop) for i in range(self.pop.N)]
        for i, j in zip(wa, wb):
            self.assertEqual(i, j)


if __name__ == "__main__":
    unittest.main()



#
# Copyright (C) 2017 Kevin Thornton <krthornt@uci.edu>
#
# This file is part of fwdpy11.
#
# fwdpy11 is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# fwdpy11 is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with fwdpy11.  If not, see <http://www.gnu.org/licenses/>.
#


def evolve(rng, pop, params, simplification_interval, recorder=None,
           suppress_table_indexing=False, record_gvalue_matrix=False,
           stopping_criterion=None,
           track_mutation_counts=False,
           remove_extinct_variants=True):
    """
    Evolve a population

    :param rng: random number generator
    :type rng: :class:`fwdpy11.GSLrng`
    :param pop: A population
    :type pop: :class:`fwdpy11.SlocusPop`
    :param params: simulation parameters
    :type params: :class:`fwdpy11.model_params.ModelParams`
    :param simplification_interval: Number of generations between simplifications.
    :type simplification_interval: int
    :param recorder: (None) A temporal sampler/data recorder.
    :type recorder: callable
    :param suppress_table_indexing: (False) Prevents edge table indexing until end of simulation
    :type suppress_table_indexing: boolean
    :param record_gvalue_matrix: (False) Whether to record genetic values into :attr:`fwdpy11.Population.genetic_values`.
    :type record_gvalue_matrix: boolean

    The recording of genetic values into :attr:`fwdpy11.Population.genetic_values` is supprssed by default.  First, it
    is redundant with :attr:`fwdpy11.DiploidMetadata.g` for the common case of mutational effects on a single trait.
    Second, we save some memory by not tracking these matrices.  However, it is useful to track these data for some
    cases when simulating multivariate mutational effects (pleiotropy).

    .. note::
        If recorder is None,
        then :class:`fwdpy11.tsrecorders.NoAncientSamples` will be used.

    """
    import fwdpy11.SlocusPop
    import fwdpy11.MlocusPop
    import warnings

    # Currently, we do not support simulating neutral mutations
    # during tree sequence simulations, so we make sure that there
    # are no neutral regions/rates:
    if len(params.nregions) != 0:
        raise ValueError(
            "Simulation of neutral mutations on tree sequences not supported (yet).")

    # Test parameters while suppressing warnings
    with warnings.catch_warnings():
        warnings.simplefilter("ignore")
        # Will throw exception if anything is wrong:
        params.validate()

    if recorder is None:
        from fwdpy11.tsrecorders import NoAncientSamples
        recorder = NoAncientSamples()

    if stopping_criterion is None:
        from ._tsevolution import _no_stopping
        stopping_criterion = _no_stopping

    from fwdpy11 import MutationRegions
    from fwdpy11 import RecombinationRegions
    if pop.__class__ is fwdpy11.SlocusPop:
        from fwdpy11 import GeneralizedGeneticMap
        from ._tsevolution import WFSlocusPop_ts
        # TODO: update to allow neutral mutations
        pneutral = 0
        mm = MutationRegions.create(pneutral, params.nregions, params.sregions)
        if all([i.__class__ is fwdpy11.Region for i in params.recregions]) is True:
            rm = RecombinationRegions(params.recrate, params.recregions)
        else:
            rm = GeneralizedGeneticMap(params.recregions)

        from fwdpy11.tsrecorders import SampleRecorder
        sr = SampleRecorder()
        WFSlocusPop_ts(rng, pop, sr, simplification_interval,
                       params.demography, params.mutrate_s,
                       mm, rm, params.gvalue,
                       recorder, stopping_criterion,
                       params.pself, params.prune_selected is False,
                       suppress_table_indexing, record_gvalue_matrix,
                       track_mutation_counts,
                       remove_extinct_variants)
    else:
        from ._tsevolution import WFMlocusPop_ts
        from fwdpy11 import MlocusMutationRegions
        from fwdpy11 import MlocusRecombinationRegions
        mm = MlocusMutationRegions()
        # The user is allowed to input an empty list
        # for nregions.  However, to make our lives
        # sane, we need to detect that case
        # and create a proxy object of a list of empty
        # lists
        nregion_proxy = params.nregions
        if len(nregion_proxy) == 0:
            nregion_proxy = [[]]*len(params.sregions)
        for n, s, i, j in zip(params.mutrate_n,
                              params.mutrate_s,
                              nregion_proxy,
                              params.sregions):
            if n != 0.0:
                raise ValueError("neutral mutations not allowed")
            temp = MutationRegions.create(n, i, j)
            mm.append(temp)
        rm = MlocusRecombinationRegions()
        for i, j in zip(params.recrates, params.recregions):
            rm.append(RecombinationRegions(i, j))
        if recorder is None:
            from fwdpy11.temporal_samplers import RecordNothing
            recorder = RecordNothing()
        from fwdpy11.tsrecorders import SampleRecorder
        sr = SampleRecorder()
        WFMlocusPop_ts(rng, pop, sr, simplification_interval,
                       params.demography, params.mutrate_s,
                       mm, rm,
                       params.interlocus_rec, params.gvalue,
                       recorder, stopping_criterion,
                       params.pself, params.prune_selected is False,
                       suppress_table_indexing, record_gvalue_matrix,
                       track_mutation_counts,
                       remove_extinct_variants)



#
# Copyright(C) 2017 Kevin Thornton < krthornt @uci.edu>
#
# This file is part of fwdpy11.
#
# fwdpy11 is free software : you can redistribute it and / or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# fwdpy11 is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with fwdpy11.If not, see < http: //www.gnu.org/licenses/>.
#
from fwdpy11._Populations import _SlocusPop


class SlocusPop(_SlocusPop):
    """
    Population object representing a single
    locus.
    """
    pass

    @staticmethod
    def create(diploids, gametes, mutations, *args):
        """
        Create a new object from input data.
        Unlike the constructor method, this method results
        in no temporary copies of input data.

        :param diplods: A :class:`fwdpy11.VecDiploid`
        :param gametes: A :class:`fwdpy11.VecGamete`
        :param mutations: A :class:`fwdpy11.VecMutation`
        :param args: Fixations, fixation times, and generation

        :rtype: :class:`fwdpy11.SlocusPop`

        See :ref:`popobjects` for example use.

        When passing in extra args, they must be the following:

        fixations: A :class:`fwdpy11.VecMutation`
        fixation times: A :class:`fwdpy11.VecUint32`
        generation: A non-negative integer

        It is required that len(fixations) == len(fixation times).

        The result of passing in these extra args will be an object
        with its fixation data populated and its generation set
        to the input value.

        .. versionadded:: 0.1.4

        """
        return SlocusPop(_SlocusPop.create(diploids,
                                           gametes, mutations,
                                           args))

    @staticmethod
    def create_from_tskit(ts):
        """
        Create a new object from an tskit.TreeSequence

        :param ts: A tree sequence from tskit
        :type ts: tskit.TreeSequence

        :rtype: :class:`fwdpy11.SlocusPop`
        :returns: A population object with an initialized
        :class:`fwdpy11.ts.TableCollection`

        .. versionadded:: 0.2.0

        .. note::

            In general, initializing a population using
            the output from a coalescent simulation is
            a tricky business.  There are issues of
            parameter scaling and the appropriateness
            of the coalescent model itself. A key issue
            is that your input tree sequence must have
            node times in the correct time units! (Generations,
            for example.) See :ref:`ts` for more discussion

        """
        from fwdpy11.ts_from_tskit import _create_SlocusPop
        return SlocusPop(_create_SlocusPop(ts))

    def dump_tables_to_tskit(self):
        """
        Dump the population's TableCollection into
        an tskit TreeSequence

        :rtype: tskit.TreeSequence

        .. todo::

            Incorporate the various metadata values.
        """
        import fwdpy11._tables_to_tskit as t2msp
        return t2msp.dump_tables_to_tskit(self)
